
using Gtk;

namespace Lsc.Widgets {
    public class Category : Object {
        public string image;
        public string name;
        
        public Category (string image, string group) {
            this.image = image;
            name = group;
        }
    }
    
    public class CategoryButton : EventBox {
        public CategoryButton (Category desc, bool fill = false) {
            sensitive = ! fill;
            visible_window = false;
            
            if (desc.image != "" && ! fill) {
                Box container = new Box(Orientation.HORIZONTAL, 3);
                container.border_width = 5;
                Box label_box = new Box(Orientation.VERTICAL, 0);
                label_box.valign = Align.CENTER;
                Image image_widget = new Image.from_icon_name(desc.image, IconSize.DND);
                
                Label title = new Label("<b>"+Utils.nicer_pkg_name(desc.name)+"</b>");
                title.ellipsize = Pango.EllipsizeMode.END;
                title.halign = Align.START;
                title.valign = Align.END;
                title.use_markup = true;
                
                Label description = new Label("<i>Comment</i>");
                description.ellipsize = Pango.EllipsizeMode.END;
                description.halign = Align.START;
                description.valign = Align.START;
                description.use_markup = true;
                
                label_box.pack_start(title, false, false, 0);
                label_box.pack_start(description, false, false, 0);
                
                container.pack_start(image_widget, false, false, 0);
                container.pack_start(label_box, false, false, 0);
                add(container);
            }
        }
    }
    
    public class CategoriesView : RoundBox {
        public signal void category_choosed (string group);
        
        // Vars
        private Box box_child;
        private Box main;
        private Separator separator;
        private CategoryButton button_child;
        public int columns { get; set; }
        private int actual_col;
        private new List<Category> children = null;
        
        private void pack_separator () {
            separator = new Separator(Orientation.HORIZONTAL);
            separator.margin_left = 1;
            separator.margin_right = 1;
            
            pack_start(separator, false, false, 0);
        }
        
        public void add_category (string image, string group) {
            children.append(new Category(image, group));
        }
        
        public void reconfigure_grid () {
            foreach (Widget widget in main.get_children()) {
                main.remove(widget);
            }
            box_child = null;
            actual_col = columns;
            
            foreach (Category button_desc in children) {
                if (actual_col == columns) {
                    box_child = new Box(Orientation.HORIZONTAL, 5);
                    box_child.homogeneous = true;
                    box_child.show();
                    main.pack_start(box_child, true, true, 0);
                    actual_col = 0;
                }
                button_child = new CategoryButton(button_desc);
                button_child.button_press_event.connect(() => {
                    category_choosed("cat");
                    return true;
                });
            
                box_child.pack_start(button_child, true, true, 0);
                button_child.show_all();
                
                actual_col++;
            }
            
            while (actual_col != columns) {
                button_child = new CategoryButton(new Category("", ""), true);
                box_child.pack_start(button_child, true, true, 0);
                button_child.show_all();
                actual_col++;
            }
        }
        
        public CategoriesView () {
            base(Orientation.VERTICAL, 0);
            columns = 0;
            actual_col = 0;
            
            main = new Box(Orientation.VERTICAL, 5);
            main.border_width = 5;
            
            box_child = new Box(Orientation.HORIZONTAL, 5);
            box_child.homogeneous = true;
            main.pack_start(box_child, true, true, 0);
            
            Label title = new Label("<b>Categories</b>");
            title.use_markup = true;
            title.halign = Align.START;
            title.margin = 5;
            
            pack_start(title, false, false, 0);
            pack_separator();
            pack_start(main, false, false, 0);
            
            notify["columns"].connect(reconfigure_grid);
        }
    }
}
